"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const vscode_uri_1 = require("vscode-uri");
const theme_check_common_1 = require("@shopify/theme-check-common");
const diagnostics_1 = require("../../diagnostics");
const documents_1 = require("../../documents");
const FixAllProvider_1 = require("./FixAllProvider");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
(0, vitest_1.describe)('Unit: FixAllProvider', () => {
    const uri = vscode_uri_1.URI.file('/path/to/file.liquid').toString();
    const contents = `
    {% assign x = 1 %}
    <script src="2.js"></script>
    <script src="3.js"></script>
  `;
    const version = 0;
    const document = vscode_languageserver_textdocument_1.TextDocument.create(uri, 'liquid', version, contents);
    let documentManager;
    let diagnosticsManager;
    let fixProvider;
    function makeOffense(checkName, needle, unfixable) {
        const start = contents.indexOf(needle);
        const end = start + needle.length;
        return {
            type: theme_check_common_1.SourceCodeType.LiquidHtml,
            check: checkName,
            message: 'Parser blocking script detected',
            absolutePath: '/path/to/file.liquid',
            severity: theme_check_common_1.Severity.ERROR,
            start: { ...document.positionAt(start), index: start },
            end: { ...document.positionAt(end), index: end },
            fix: unfixable ? undefined : (corrector) => corrector.replace(18, 36, 'fixed'),
        };
    }
    (0, vitest_1.beforeEach)(() => {
        documentManager = new documents_1.DocumentManager();
        diagnosticsManager = new diagnostics_1.DiagnosticsManager({ sendDiagnostics: vitest_1.vi.fn() });
        documentManager.open(uri, contents, version);
        fixProvider = new FixAllProvider_1.FixAllProvider(documentManager, diagnosticsManager);
    });
    (0, vitest_1.it)('provides a code action to solve all fixable problems', async () => {
        diagnosticsManager.set(uri, version, [
            makeOffense('UnusedAssign', '{% assign x = 1 %}'),
            makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
        ]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('2.js')),
                end: document.positionAt(contents.indexOf('2.js')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(1);
        const [fixAllAction] = codeActions;
        (0, vitest_1.expect)(fixAllAction).toEqual({
            title: 'Fix all auto-fixable problems',
            kind: 'source.fixAll',
            diagnostics: vitest_1.expect.any(Array),
            isPreferred: false,
            command: {
                title: 'applyFixes',
                command: 'themeCheck/applyFixes',
                arguments: [uri, version, [0, 1, 2]],
            },
        });
    });
    (0, vitest_1.it)('does not provide code actions for non-fixable offenses', async () => {
        const nonFixableOffense = makeOffense('ParserBlockingScript', '<script src="2.js">', true);
        diagnosticsManager.set(uri, version, [nonFixableOffense]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('2.js')),
                end: document.positionAt(contents.indexOf('2.js')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(0);
    });
    (0, vitest_1.it)('does provide code action if the selection range does not overlap with the error', async () => {
        diagnosticsManager.set(uri, version, [
            makeOffense('ParserBlockingScript', '<script src="2.js"></script>'),
            makeOffense('ParserBlockingScript', '<script src="3.js"></script>'),
        ]);
        const codeActions = fixProvider.codeActions({
            textDocument: { uri },
            range: {
                start: document.positionAt(contents.indexOf('assign x = 1')),
                end: document.positionAt(contents.indexOf('= 1')),
            },
            context: { diagnostics: [] },
        });
        (0, vitest_1.expect)(codeActions.length).toBe(1);
    });
});
//# sourceMappingURL=FixAllProvider.spec.js.map