"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectCompletionProvider = void 0;
const liquid_html_parser_1 = require("@shopify/liquid-html-parser");
const vscode_languageserver_1 = require("vscode-languageserver");
const visitor_1 = require("../../visitor");
const params_1 = require("../params");
const common_1 = require("./common");
class ObjectCompletionProvider {
    constructor(themeDocset) {
        this.themeDocset = themeDocset;
    }
    async completions(params) {
        if (!params.completionContext)
            return [];
        const { partialAst, node, ancestors } = params.completionContext;
        if (!node || node.type !== liquid_html_parser_1.NodeTypes.VariableLookup) {
            return [];
        }
        if (!node.name || node.lookups.length > 0) {
            // We only do top level in this one.
            return [];
        }
        const partial = node.name.replace(params_1.CURSOR, '');
        const globals = await this.themeDocset.objects();
        const context = collectVariables(partialAst, ancestors);
        const options = globals.concat(context);
        return options
            .filter(({ name }) => name.startsWith(partial))
            .sort(common_1.sortByName)
            .map((tag) => (0, common_1.createCompletionItem)(tag, { kind: vscode_languageserver_1.CompletionItemKind.Variable }, 'object'));
    }
}
exports.ObjectCompletionProvider = ObjectCompletionProvider;
// I want the names of the things that are in range.
function collectVariables(partialAst, currentAncestors) {
    const entries = (0, visitor_1.visit)(partialAst, {
        // {% assign x = foo %}
        AssignMarkup(node) {
            // Can't use x in {% assign x = x %}
            if (currentAncestors.includes(node)) {
                return;
            }
            return node.name;
        },
        // This also covers tablerow
        ForMarkup(node, ancestors) {
            const parentNode = ancestors.at(-1);
            // We can't do {% for a in a %}
            // Nor {% for a in a limit: a %}
            if (currentAncestors.includes(node)) {
                return;
            }
            // A for loop is out of scope if it is closed.
            // {% for el in col %}
            //   ...
            // {% endfor %}
            // {% # el is undefined here %}
            // {% echo █ %}
            const isOutOfScope = parentNode.blockEndPosition && parentNode.blockEndPosition.end !== -1;
            if (isOutOfScope)
                return;
            return node.variableName;
        },
        // {% capture foo %}
        //   ...
        // {% endcapture}
        LiquidTag(node) {
            if (node.name === 'capture' && typeof node.markup !== 'string') {
                return node.markup.name;
            }
        },
    });
    return entries.map((name) => ({ name }));
}
//# sourceMappingURL=ObjectCompletionProvider.js.map