"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const startServer_1 = require("./startServer");
const MockConnection_1 = require("../test/MockConnection");
const vscode_languageserver_1 = require("vscode-languageserver");
const theme_check_common_1 = require("@shopify/theme-check-common");
const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));
(0, vitest_1.describe)('Module: server', () => {
    const filePath = 'snippets/code.liquid';
    const fileURI = `browser:///${filePath}`;
    const fileContents = `{% render 'foo' %}`;
    let connection;
    let dependencies;
    let fileTree;
    let logger;
    (0, vitest_1.beforeEach)(() => {
        // Initialize all ze mocks...
        connection = (0, MockConnection_1.mockConnection)();
        fileTree = new Set(['snippets/code.liquid']);
        logger = vitest_1.vi.fn();
        dependencies = getDependencies(logger, fileTree);
        // Start the server
        (0, startServer_1.startServer)(connection, dependencies);
        // Perform the initialize/initialized setup steps
        connection.setup();
        // Stop the time
        vitest_1.vi.useFakeTimers();
    });
    (0, vitest_1.afterEach)(() => {
        vitest_1.vi.useRealTimers();
    });
    (0, vitest_1.it)("should log Let's roll! on successful setup", () => {
        (0, vitest_1.expect)(logger).toHaveBeenCalledWith("[SERVER] Let's roll!");
    });
    (0, vitest_1.it)('should debounce calls to runChecks', async () => {
        connection.openDocument(filePath, `{% echo 'hello' %}`);
        connection.changeDocument(filePath, `{% echo 'hello w' %}`, 1);
        connection.changeDocument(filePath, `{% echo 'hello wor' %}`, 2);
        connection.changeDocument(filePath, `{% echo 'hello world' %}`, 3);
        await flushAsync();
        // Make sure nothing was sent
        (0, vitest_1.expect)(connection.spies.sendNotification).not.toHaveBeenCalled();
        // Advance time by debounce time
        await advanceAndFlush(100);
        // Make sure you get the diagnostics you'd expect (for the right
        // version of the file)
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledOnce();
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            diagnostics: [],
            uri: fileURI,
            version: 3,
        });
    });
    (0, vitest_1.it)('should trigger a re-check on did create files notifications', async () => {
        // Setup & expectations
        connection.openDocument(filePath, fileContents);
        await advanceAndFlush(100);
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            uri: fileURI,
            version: 0,
            diagnostics: [missingTemplateDiagnostic()],
        });
        // Clear mocks for future use
        connection.spies.sendNotification.mockClear();
        // Trigger create files notification & update mocks
        connection.triggerNotification(vscode_languageserver_1.DidCreateFilesNotification.type, {
            files: [
                {
                    uri: 'browser:///snippets/foo.liquid',
                },
                {
                    uri: 'browser:///snippets/bar.liquid',
                },
            ],
        });
        fileTree.add('/snippets/foo.liquid');
        fileTree.add('/snippets/bar.liquid');
        await advanceAndFlush(100);
        // Verify that we re-check'ed filePath to remove the linting error
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledOnce();
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            diagnostics: [],
            uri: fileURI,
            version: 0,
        });
    });
    (0, vitest_1.it)('should trigger a re-check on did file rename notifications', async () => {
        // Setup & expectations
        fileTree.add('/snippets/bar.liquid');
        connection.openDocument(filePath, fileContents);
        await advanceAndFlush(100);
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            uri: fileURI,
            version: 0,
            diagnostics: [missingTemplateDiagnostic()],
        });
        // Reset mocks for different expectations later
        connection.spies.sendNotification.mockClear();
        // Trigger a file rename notification
        connection.triggerNotification(vscode_languageserver_1.DidRenameFilesNotification.type, {
            files: [
                {
                    oldUri: 'browser:///snippets/bar.liquid',
                    newUri: 'browser:///snippets/foo.liquid',
                },
            ],
        });
        // Adjust mocks
        fileTree.delete('/snippets/bar.liquid');
        fileTree.add('/snippets/foo.liquid');
        // Advance time
        await advanceAndFlush(100);
        // Make sure only one publishDiagnostics has been called and that the
        // error disappears because of the file rename.
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledOnce();
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            diagnostics: [],
            uri: fileURI,
            version: 0,
        });
    });
    (0, vitest_1.it)('should trigger a re-check on did delete files notifications', async () => {
        // Setup and expectations (no errors)
        fileTree.add('/snippets/foo.liquid');
        connection.openDocument(filePath, fileContents);
        await advanceAndFlush(100);
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            uri: fileURI,
            version: 0,
            diagnostics: [],
        });
        // Clear mocks for future expectations
        connection.spies.sendNotification.mockClear();
        // Notify about file delete
        connection.triggerNotification(vscode_languageserver_1.DidDeleteFilesNotification.type, {
            files: [
                {
                    uri: 'browser:///snippets/foo.liquid',
                },
            ],
        });
        fileTree.delete('/snippets/foo.liquid');
        await advanceAndFlush(100);
        // Make sure there's an error now that the file no longer exists
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledOnce();
        (0, vitest_1.expect)(connection.spies.sendNotification).toHaveBeenCalledWith(vscode_languageserver_1.PublishDiagnosticsNotification.type, {
            diagnostics: [missingTemplateDiagnostic()],
            uri: fileURI,
            version: 0,
        });
    });
    // When you're using fake timers and stuff runs async, you want to flush
    // the async stuff that would happen on a timer.
    //
    // We can't simply `await sleep(1)` because the timer is stopped, so we
    // do this Promise.all thing here that does both.
    function flushAsync() {
        return Promise.all([vitest_1.vi.advanceTimersByTimeAsync(1), sleep(1)]);
    }
    function advanceAndFlush(ms) {
        vitest_1.vi.advanceTimersByTime(ms);
        return flushAsync();
    }
    function getDependencies(logger, fileTree) {
        const MissingTemplate = theme_check_common_1.allChecks.filter((c) => c.meta.code === 'MissingTemplate');
        return {
            findRootURI: async (_) => 'browser:///',
            fileExists: vitest_1.vi
                .fn()
                .mockImplementation(async (absolutePath) => fileTree.has(absolutePath)),
            fileSize: vitest_1.vi.fn().mockResolvedValue(420),
            getDefaultTranslationsFactory: () => async () => ({}),
            getDefaultLocaleFactory: () => async () => 'en',
            loadConfig: async () => ({
                settings: {},
                checks: MissingTemplate,
                root: '/',
            }),
            log: logger,
            themeDocset: {
                filters: async () => [],
                objects: async () => [],
                tags: async () => [],
            },
            schemaValidators: {
                validateSectionSchema: async () => ({}),
            },
        };
    }
    function missingTemplateDiagnostic() {
        return {
            code: 'MissingTemplate',
            codeDescription: { href: vitest_1.expect.any(String) },
            message: "'snippets/foo.liquid' does not exist",
            severity: 1,
            source: 'theme-check',
            range: {
                start: {
                    character: 10,
                    line: 0,
                },
                end: {
                    character: 15,
                    line: 0,
                },
            },
        };
    }
});
//# sourceMappingURL=startServer.spec.js.map